#ifndef CODE_GENERATOR_H
#define CODE_GENERATOR_H

#include <vector>
#include <map>
#include <iostream>
#include <string>
#include <stdint.h>
#include "compiler_state.h"

///@class CodeGenerator
///@brief Encapsulates details related to code generation
class CodeGenerator
{
public:
	CodeGenerator();
	
public:
	///@method generateInstruction
	///@brief Generates code
	void generateInstruction(const std::string& str, CompilerState state);
	
	///@method dumpCode
	///@brief Finalizes the "proto-code", writes the resulting code to an output stream and resets the inner state
	///@param out The output stream
	void dumpCode(std::ostream& out);
	
	void generateSimpleInstructionCode(const std::string& str);
	void generateLabelDefCode(const std::string& str);
	void generateLabelRefCode(const std::string& str);
	void generateImmediateValueCode(const std::string& str);
	void generateCharacterConstantCode(const std::string& str);
	void generateStringConstantCode(const std::string& str);
	void generatePRINTCode();

private:
	///@method finalizeCode
	///@brief Finalizes teh generated code (inserts label addresses at the right places)
	///@throw UndefinedLabel
	void finalizeCode();

	///@method initInstructionTable
	///@brief Initialize instruction table
	void initInstructionTable();

private:
	static void convertToUppercase(std::string&);
	
private:
	///@brief Holds information about a label
	struct LabelInfo
	{
		LabelInfo():address(-1){}
		
		///@brief The adress of the label (-1 means undefined)
		int address;
		
		///@brief Addresses from where the label is referenced
		std::vector<int32_t> ref;
	};

	//Some handy typedefs
	typedef std::map<std::string, LabelInfo> LabelTable;
	typedef std::map<std::string, int32_t> InstructionTable;
	typedef std::vector<int32_t> Code;

private:
	///@brief Holds the generated code
	Code m_code;
	
	///@brief Maps label names to label information
	LabelTable m_labels;
	
	///@brief Maps insruction names to instruction codes
	InstructionTable m_instr;	
};

#endif //CODE_GENERATOR_H
